@@ -1,4 +1,6 @@
 import logging
+
+import foldedleastsquares
 import lightkurve as lk
 import numpy as np
 import pandas as pd
@@ -8,6 +10,9 @@ import plotly.graph_objects as go
 import pylab
 import wotan
 from astropy.stats import sigma_clip
+from lcbuilder.lcbuilder_class import LcBuilder
+from lcbuilder.objectinfo.MissionFfiIdObjectInfo import MissionFfiIdObjectInfo
+from lcbuilder.objectinfo.ObjectInfo import ObjectInfo
 from scipy import stats, signal
 from scipy.signal import argrelextrema, savgol_filter
 from wotan import flatten
@@ -25,65 +30,60 @@ class SherlockExplorer:
     MISSION_ID_TESS = "TIC"
 
     def explore_object(self, object_id, input_lc_file=None, time_units=1, auto_detrend_periodic_signals=False,
-                       auto_detrend_ratio=1/4, detrend_method="cosine", smooth=False, sectors=None):
-        mission, mission_id, id = self.__parse_object_id(object_id)
-        if input_lc_file is None:
-            lcf = lk.search_lightcurvefile(str(object_id), mission=mission, sector=sectors).download_all()
-            lc = lcf.stitch().remove_nans()
-            if mission_id == self.MISSION_ID_KEPLER_2:
-                lc = lc.flatten().to_corrector("sff").correct(windows=20)
-        else:
-            df = pd.read_csv(input_lc_file, float_precision='round_trip', sep=',',
-                             usecols=['#time', 'flux', 'flux_err'])
-            lc = lk.LightCurve(time=df['#time'], flux=df['flux'], flux_err=df['flux_err'])
+                       auto_detrend_ratio=1/4, detrend_method="cosine", smooth=False, sectors=None, cadence=300):
+        lc_builder = LcBuilder()
+        object_info = lc_builder.build_object_info(object_id, None, sectors, input_lc_file, cadence, None,
+                                                   None, None, None, None)
+        lc_build = \
+            lc_builder.build(object_info, ".")
+        lc = lc_build.lc.remove_outliers(sigma_lower=float('inf'), sigma_upper=3)  # remove outliers over 3sigma
         flux = lc.flux.value
         flux_err = lc.flux_err.value
         if time_units == 0:
             time = lc.astropy_time.jd
         else:
-            time = lc.time
+            time = lc.time.value
         if smooth:
+            flux = wotan.flatten(time, flux, method="biweight", window_length=0.5)
             flux = savgol_filter(flux, 11, 2)
-            flux = uniform_filter1d(flux, 11)
-        lc = lk.LightCurve(time=time, flux=flux, flux_err=flux_err)
-        lc = lc.remove_outliers(sigma_lower=float('inf'), sigma_upper=3)  # remove outliers over 3sigma
+            #flux = uniform_filter1d(flux, 11)
         lc_df = pd.DataFrame(columns=['#time', 'flux', 'flux_err'])
-        lc_df['#time'] = lc.time.value.astype('<f4')
-        lc_df['flux'] = lc.flux.value.astype('<f4')
+        lc_df['#time'] = lc.time.value
+        lc_df['flux'] = flux
         lc_df['flux_err'] = lc.flux_err.value.astype('<f4')
         lc_df = pd.DataFrame(columns=['flux'])
-        lc_df['flux'] = acf(lc.flux.value.astype('<f4'), nlags=7200)
+        lc_df['flux'] = acf(flux, nlags=7200)
         test_df = pd.DataFrame(lc_df)
         ax = test_df.plot()
-        ax.set_ylim(-0.1, 0.1)
+        ax.set_ylim(lc_df["flux"].min(), lc_df["flux"].max())
         #ax.set_xticks(list(range(0, len(ax.get_xticks()))), ax.get_xticks() / 30 / 24)
         plt.show()
         #periodogram = lc.to_periodogram(oversample_factor=10)
         #fig = px.line(x=periodogram.period.astype('<f4'), y=periodogram.power.astype('<f4'), log_x=True)
         #fig.show()
-        fig = px.scatter(x=lc.time.value.astype('<f4'), y=lc.flux.value.astype('<f4'))
+        fig = px.scatter(x=time, y=flux)
         fig.show()
         if auto_detrend_periodic_signals:
             detrend_period = self.__calculate_max_significant_period(lc, periodogram)
             if detrend_period is not None:
                 flatten_lc, trend_lc = self.__detrend_by_period(lc, detrend_period * auto_detrend_ratio, detrend_method)
                 fig = go.Figure()
-                fig.add_trace(go.Scatter(x=lc.time.astype('<f4'), y=flux.astype('<f4'), mode='markers', name='Flux'))
-                fig.add_trace(go.Scatter(x=lc.time.astype('<f4'), y=trend_lc.astype('<f4'), mode='lines+markers',
+                fig.add_trace(go.Scatter(x=lc.time.value.astype('<f4'), y=flux.astype('<f4'), mode='markers', name='Flux'))
+                fig.add_trace(go.Scatter(x=lc.time.value.astype('<f4'), y=trend_lc.astype('<f4'), mode='lines+markers',
                                          name='Main Trend'))
                 fig.show()
                 if auto_detrend_periodic_signals:
-                    fig = px.line(x=lc.time.astype('<f4'), y=flatten_lc.astype('<f4'))
+                    fig = px.line(x=lc.time.value.astype('<f4'), y=flatten_lc.astype('<f4'))
                     fig.show()
                     lc.flux = flatten_lc
-        bin_means, bin_edges, binnumber = stats.binned_statistic(lc.time.value, lc.flux.value, statistic='mean',bins=len(time) / 5)
+        bin_means, bin_edges, binnumber = stats.binned_statistic(time, flux, statistic='mean',bins=len(time) / 5)
         bin_stds, _, _ = stats.binned_statistic(time, flux, statistic='std', bins=len(time) / 3)
         bin_width = (bin_edges[1] - bin_edges[0])
         bin_centers = bin_edges[1:] - bin_width / 2
-        time_binned = bin_centers.astype('<f4')
-        flux_binned = bin_means.astype('<f4')
+        time_binned = bin_centers
+        flux_binned = bin_means
         lc_binned = lk.LightCurve(time=time_binned, flux=flux_binned)
-        fig = px.scatter(x=lc.time.value.astype('<f4'), y=lc.flux.value.astype('<f4'))
+        fig = px.scatter(x=lc_binned.time.value, y=lc_binned.flux.value)
         fig.show()
         while True:
             try:
@@ -109,25 +109,25 @@ class SherlockExplorer:
             # args_flux_outer = np.argwhere(flux < 1 + depth * 0.001)
             # flux[args_flux_outer] = np.nan
             # time[args_flux_outer] = np.nan
-            folded_lc = lk.LightCurve(time=lc.time, flux=lc.flux, flux_err=lc.flux_err)
-            folded_lc.remove_nans()
-            folded_lc = lc.fold(t0=t0, period=period)
-            folded_lc.flux_err = np.nan
-            #folded_lc = folded_lc.bin(bins=500)
+
             j = 0
-            fig_transit, axs = plt.subplots(3, 1, figsize=(8, 8))
+            fig_transit, axs = plt.subplots(4, 4, figsize=(16, 16))
             #fig_transit.suptitle("Transit centered plots for TIC " + str(object_id))
-            for i in np.arange(t0, t0 + period * 3, period)[0:3]:
+            for i in np.arange(t0, t0 + period * 16, period)[0:16]:
                 args_plot = np.argwhere((i - 0.1 < time) & (time < i + 0.1)).flatten()
                 plot_time = time[args_plot]
                 plot_flux = flux[args_plot]
-                axs[j].scatter(plot_time, plot_flux, color='gray', alpha=1, rasterized=True, label="Flux Transit " + str(j))
-                axs[j].set_title("Transit " + str(j))
-                axs[j].set_xlabel('Time')
-                axs[j].set_ylabel('Flux')
+                axs[j // 4][j % 4].scatter(plot_time, plot_flux, color='gray', alpha=1, rasterized=True, label="Flux Transit " + str(j))
                 j = j + 1
+            folded_lc = lk.LightCurve(time=lc.time, flux=lc.flux, flux_err=lc.flux_err)
+            folded_lc.remove_nans()
+            # folded_lc.flux_err = np.nan
+            # folded_lc = folded_lc.bin(bins=500)
+            folded_lc.fold(period=period).scatter()
+            plt.title("Phase-folded period: " + format(period, ".2f") + " days")
+            plt.show()
             fig_transit.show()
-            fig = px.scatter(x=folded_lc.phase.astype('<f4'), y=folded_lc.flux.astype('<f4'))
+            fig = px.line(x=folded_lc.time.value.astype('<f4'), y=folded_lc.flux.value.astype('<f4'))
             fig.show()
 
     def __parse_object_id(self, object_id):
@@ -165,10 +165,10 @@ class SherlockExplorer:
 
     def __detrend_by_period(self, lc, period_window, detrend_method):
         if detrend_method == 'gp':
-            flatten_lc, lc_trend = flatten(lc.time, lc.flux, method=detrend_method, kernel='matern',
+            flatten_lc, lc_trend = flatten(lc.time.value, lc.flux.value, method=detrend_method, kernel='matern',
                                    kernel_size=period_window, return_trend=True, break_tolerance=0.5)
         else:
-            flatten_lc, lc_trend = flatten(lc.time, lc.flux, window_length=period_window, return_trend=True,
+            flatten_lc, lc_trend = flatten(lc.time.value, lc.flux.value, window_length=period_window, return_trend=True,
                                    method=detrend_method, break_tolerance=0.5)
         flatten_lc = sigma_clip(flatten_lc, sigma_lower=20, sigma_upper=3)
         return flatten_lc, lc_trend
