@@ -1,7 +1,9 @@
+import copy
 import os
 import shutil
 from argparse import ArgumentParser
 
+import allesfitter
 import astroplan
 import matplotlib
 from astroplan.plots import plot_airmass
@@ -13,7 +15,7 @@ import numpy as np
 from astroplan import (Observer, FixedTarget, AtNightConstraint, AltitudeConstraint)
 import matplotlib.pyplot as plt
 
-from astropy.time import Time
+from astropy.time import Time, TimeDelta
 from lcbuilder.lcbuilder_class import LcBuilder
 from pytz import timezone, utc
 from timezonefinder import TimezoneFinder
@@ -92,14 +94,15 @@ def get_offset(lat, lng, datetime):
     return (today_utc - today_target).total_seconds() / 3600
 
 
-def create_observation_observables(object_id, object_dir, name, epoch, epoch_low_err, epoch_up_err, period,
+def create_observation_observables(object_id, object_dir, since, name, epoch, epoch_low_err, epoch_up_err, period,
                                    period_low_err, period_up_err, duration,
                                    observatories_file, timezone, latitude, longitude, altitude,
-                                   max_days, min_altitude, moon_min_dist, moon_max_dist, transit_fraction):
+                                   max_days, min_altitude, moon_min_dist, moon_max_dist, transit_fraction, baseline):
     """
 
     @param object_id: the candidate id
     @param object_dir: the candidate directory
+    @param since: starting plan date
     @param name: the name given to the candidate
     @param epoch: the candidate epoch
     @param epoch_low_err: the candidate epoch's lower error
@@ -119,10 +122,11 @@ def create_observation_observables(object_id, object_dir, name, epoch, epoch_low
     @param moon_max_dist: the minimum moon distance for moon illumination = 1
     @param transit_fraction: the minimum transit observability (0.25 for at least ingress/egress, 0.5 for ingress/egress
     + midtime, 1 for ingress, egress and midtime).
+    @param baseline: the required baseline in hours.
     @return: the generated data and target folders
     """
     if observatories_file is not None:
-        observatories_df = pd.read_csv(observatories_file)
+        observatories_df = pd.read_csv(observatories_file, comment='#')
     else:
         observatories_df = pd.DataFrame(columns=['name', 'tz', 'lat', 'long', 'alt'])
         observatories_df = observatories_df.append("Obs-1", timezone, latitude, longitude, altitude)
@@ -136,12 +140,10 @@ def create_observation_observables(object_id, object_dir, name, epoch, epoch_low
         primary_eclipse_time = Time(epoch, format='jd')
     target = FixedTarget(SkyCoord(coords, unit=(u.deg, u.deg)))
     n_transits = max_days // period
-    obs_time = Time.now()
     system = EclipsingSystem(primary_eclipse_time=primary_eclipse_time, orbital_period=u.Quantity(period, unit="d"),
                              duration=u.Quantity(duration, unit="h"), name=name)
-    observables_df = pd.DataFrame(columns=['observatory', 'timezone', 'ingress', 'egress', 'midtime',
-                                           "midtime_up_err_h",
-                                           "midtime_low_err_h", 'twilight_evening',
+    observables_df = pd.DataFrame(columns=['observatory', 'timezone', 'start_obs', 'end_obs', 'ingress', 'egress',
+                                           'midtime', "midtime_up_err_h", "midtime_low_err_h", 'twilight_evening',
                                            'twilight_morning', 'observable', 'moon_phase', 'moon_dist'])
     plan_dir = object_dir + "/plan"
     images_dir = plan_dir + "/images"
@@ -151,48 +153,59 @@ def create_observation_observables(object_id, object_dir, name, epoch, epoch_low
     if os.path.exists(images_dir):
         shutil.rmtree(images_dir, ignore_errors=True)
     os.mkdir(images_dir)
+    alert_date = None
     for index, observatory_row in observatories_df.iterrows():
         observer_site = Observer(latitude=observatory_row["lat"], longitude=observatory_row["lon"],
                                  elevation=u.Quantity(observatory_row["alt"], unit="m"))
-        midtransit_times = system.next_primary_eclipse_time(obs_time, n_eclipses=n_transits)
-        ingress_egress_times = system.next_primary_ingress_egress_time(obs_time, n_eclipses=n_transits)
+        midtransit_times = system.next_primary_eclipse_time(since, n_eclipses=n_transits)
+        ingress_egress_times = system.next_primary_ingress_egress_time(since, n_eclipses=n_transits)
         constraints = [AtNightConstraint.twilight_nautical(), AltitudeConstraint(min=min_altitude * u.deg),
                        MoonIlluminationSeparationConstraint(min_dist=moon_min_dist * u.deg,
                                                             max_dist=moon_max_dist * u.deg)]
-        ingress_observables = astroplan.is_event_observable(constraints, observer_site, target, times=ingress_egress_times[:, 0])[0]
-        midtime_observables = astroplan.is_event_observable(constraints, observer_site, target, times=midtransit_times)[0]
-        egress_observables = astroplan.is_event_observable(constraints, observer_site, target, times=ingress_egress_times[:, 1])[0]
-        entire_observables = astroplan.is_event_observable(constraints, observer_site, target, times_ingress_egress=ingress_egress_times)[0]
         moon_for_midtransit_times = get_moon(midtransit_times)
         moon_dist_midtransit_times = moon_for_midtransit_times.separation(
             SkyCoord(star_df.iloc[0]["ra"], star_df.iloc[0]["dec"], unit="deg"))
         moon_phase_midtransit_times = np.round(astroplan.moon_illumination(midtransit_times), 2)
+        transits_since_epoch = np.round((midtransit_times - primary_eclipse_time).jd / period)
+        midtransit_time_low_err = np.round(
+            (((transits_since_epoch * period_low_err) ** 2 + epoch_low_err ** 2) ** (1 / 2)) * 24, 2)
+        midtransit_time_up_err = np.round(
+            (((transits_since_epoch * period_up_err) ** 2 + epoch_up_err ** 2) ** (1 / 2)) * 24, 2)
+        low_err_delta = TimeDelta(midtransit_time_low_err * 3600, format='sec')
+        up_err_delta = TimeDelta(midtransit_time_up_err * 3600, format='sec')
         i = 0
         for midtransit_time in midtransit_times:
-            ingress = ingress_egress_times[i][0]
-            egress = ingress_egress_times[i][1]
-            ingress_observable = ingress_observables[i]
-            egress_observable = egress_observables[i]
-            midtime_observable = midtime_observables[i]
-            entire_observable = entire_observables[i]
-            moon_dist = round(moon_dist_midtransit_times[i].degree)
-            observable = 0
-            if ingress_observable and egress_observable:
-                observable = 1
-            elif (ingress_observable and midtime_observable) or (midtime_observable and egress_observable):
-                observable = 0.5
-            elif ingress_observable or egress_observable:
-                observable = 0.25
-            if observable < transit_fraction:
-                i = i + 1
-                continue
             twilight_evening = observer_site.twilight_evening_nautical(midtransit_time)
             twilight_morning = observer_site.twilight_morning_nautical(midtransit_time)
+            ingress = ingress_egress_times[i][0]
+            egress = ingress_egress_times[i][1]
+            lowest_ingress = ingress - low_err_delta[i]
+            highest_egress = egress + up_err_delta[i]
+            if (highest_egress - lowest_ingress).jd > 0.33:
+                alert_date = midtransit_time if (alert_date is None) or (alert_date is not None and alert_date >= midtransit_time) else alert_date
+                break
+            else:
+                baseline_low = lowest_ingress - baseline * u.hour
+                baseline_up = highest_egress + baseline * u.hour
+                transit_times = baseline_low + (baseline_up - baseline_low) * np.linspace(0, 1, 100)
+                observable_transit_times = astroplan.is_event_observable(constraints, observer_site, target,
+                                                                   times=transit_times)[0]
+                observable_transit_times_true = np.argwhere(observable_transit_times)
+                observable = len(observable_transit_times_true) / 100
+                if observable < transit_fraction:
+                    i = i + 1
+                    continue
+                start_obs = transit_times[observable_transit_times_true[0]][0]
+                end_obs = transit_times[observable_transit_times_true[len(observable_transit_times_true) - 1]][0]
+                start_plot = baseline_low
+                end_plot = baseline_up
+                if twilight_evening > start_obs:
+                    start_obs = twilight_evening
+                if twilight_morning < end_obs:
+                    end_obs = twilight_morning
+            moon_dist = round(moon_dist_midtransit_times[i].degree)
             moon_phase = moon_phase_midtransit_times[i]
             # TODO get is_event_observable for several parts of the transit (ideally each 5 mins) to get the proper observable percent. Also with baseline
-            transits_since_epoch = round((midtransit_time - primary_eclipse_time).jd / period)
-            midtransit_time_low_err = np.round((((transits_since_epoch * period_low_err) ** 2 + epoch_low_err ** 2) ** (1 / 2)) * 24, 2)
-            midtransit_time_up_err = np.round((((transits_since_epoch * period_up_err) ** 2 + epoch_up_err ** 2) ** (1 / 2)) * 24, 2)
             if observatory_row["tz"] is not None and not np.isnan(observatory_row["tz"]):
                 observer_timezone = observatory_row["tz"]
             else:
@@ -200,21 +213,19 @@ def create_observation_observables(object_id, object_dir, name, epoch, epoch_low
                                                midtransit_time.datetime)
             observables_df = observables_df.append({"observatory": observatory_row["name"],
                                                     "timezone": observer_timezone, "ingress": ingress.isot,
+                                                    "start_obs": start_obs.isot, "end_obs": end_obs.isot,
                                    "egress": egress.isot, "midtime": midtransit_time.isot,
-                                   "midtime_up_err_h": midtransit_time_up_err,
-                                   "midtime_low_err_h": midtransit_time_low_err,
+                                   "midtime_up_err_h": str(int(midtransit_time_up_err[i] // 1)) + ":" + str(int(midtransit_time_up_err[i] % 1 * 60)),
+                                   "midtime_low_err_h": str(int(midtransit_time_low_err[i] // 1)) + ":" + str(int(midtransit_time_low_err[i] % 1 * 60)),
                                    "twilight_evening": twilight_evening.isot,
                                    "twilight_morning": twilight_morning.isot,
                                    "observable": observable, "moon_phase": moon_phase, "moon_dist": moon_dist},
                                                    ignore_index=True)
-            delta_t = (egress - ingress) * 3
-            init_plot_time = ingress - duration * u.hour
-            end_plot_time = egress + duration * u.hour
-            plot_time = init_plot_time + delta_t * np.linspace(0, 1, 50)
+            plot_time = start_plot + (end_plot - start_plot) * np.linspace(0, 1, 100)
             plt.tick_params(labelsize=6)
             airmass_ax = plot_airmass(target, observer_site, plot_time, brightness_shading=False, altitude_yaxis=True)
-            airmass_ax.axvspan(twilight_morning.plot_date, end_plot_time.plot_date, color='white')
-            airmass_ax.axvspan(init_plot_time.plot_date, twilight_evening.plot_date, color='white')
+            airmass_ax.axvspan(twilight_morning.plot_date, end_plot.plot_date, color='white')
+            airmass_ax.axvspan(start_plot.plot_date, twilight_evening.plot_date, color='white')
             airmass_ax.axvspan(twilight_evening.plot_date, twilight_morning.plot_date, color='gray')
             airmass_ax.axhspan(1. / np.cos(np.radians(90 - min_altitude)), 5.0, color='green')
             airmass_ax.get_figure().gca().set_title("")
@@ -224,21 +235,39 @@ def create_observation_observables(object_id, object_dir, name, epoch, epoch_low
             airmass_ax.set_ylabel("")
             xticks = []
             xticks_labels = []
-            if ingress > init_plot_time and ingress < end_plot_time:
+            xticks.append(start_obs.plot_date)
+            hour_min_sec_arr = start_obs.isot.split("T")[1].split(":")
+            xticks_labels.append("T1_" + hour_min_sec_arr[0] + ":" + hour_min_sec_arr[1])
+            plt.axvline(x=start_obs.plot_date, color="violet")
+            xticks.append(end_obs.plot_date)
+            hour_min_sec_arr = end_obs.isot.split("T")[1].split(":")
+            xticks_labels.append("T1_" + hour_min_sec_arr[0] + ":" + hour_min_sec_arr[1])
+            plt.axvline(x=end_obs.plot_date, color="violet")
+            if lowest_ingress > start_plot and lowest_ingress < end_plot:
+                xticks.append(lowest_ingress.plot_date)
+                hour_min_sec_arr = lowest_ingress.isot.split("T")[1].split(":")
+                xticks_labels.append("T1_" + hour_min_sec_arr[0] + ":" + hour_min_sec_arr[1])
+                plt.axvline(x=lowest_ingress.plot_date, color="red")
+            if ingress > start_plot and ingress < end_plot:
                 xticks.append(ingress.plot_date)
                 hour_min_sec_arr = ingress.isot.split("T")[1].split(":")
                 xticks_labels.append("T1_" + hour_min_sec_arr[0] + ":" + hour_min_sec_arr[1])
                 plt.axvline(x=ingress.plot_date, color="orange")
-            if midtransit_time > init_plot_time and midtransit_time < end_plot_time:
+            if midtransit_time > start_plot and midtransit_time < end_plot:
                 xticks.append(midtransit_time.plot_date)
                 hour_min_sec_arr = midtransit_time.isot.split("T")[1].split(":")
                 xticks_labels.append("T0_" + hour_min_sec_arr[0] + ":" + hour_min_sec_arr[1])
                 plt.axvline(x=midtransit_time.plot_date, color="black")
-            if egress > init_plot_time and egress < end_plot_time:
+            if egress > start_plot and egress < end_plot:
                 xticks.append(egress.plot_date)
                 hour_min_sec_arr = egress.isot.split("T")[1].split(":")
                 xticks_labels.append("T4_" + hour_min_sec_arr[0] + ":" + hour_min_sec_arr[1])
                 plt.axvline(x=egress.plot_date, color="orange")
+            if highest_egress > start_plot and highest_egress < end_plot:
+                xticks.append(highest_egress.plot_date)
+                hour_min_sec_arr = highest_egress.isot.split("T")[1].split(":")
+                xticks_labels.append("T4_" + hour_min_sec_arr[0] + ":" + hour_min_sec_arr[1])
+                plt.axvline(x=highest_egress.plot_date, color="red")
             airmass_ax.xaxis.set_tick_params(labelsize=5)
             airmass_ax.set_xticks([])
             airmass_ax.set_xticklabels([])
@@ -255,7 +284,7 @@ def create_observation_observables(object_id, object_dir, name, epoch, epoch_low
     observables_df = observables_df.sort_values(["midtime", "observatory"], ascending=True)
     observables_df.to_csv(plan_dir + "/observation_plan.csv", index=False)
     print("Observation plan created in directory: " + object_dir)
-    return observatories_df, observables_df, plan_dir, images_dir
+    return observatories_df, observables_df, alert_date, plan_dir, images_dir
 
 if __name__ == '__main__':
     ap = ArgumentParser(description='Planning of observations from Sherlock objects of interest')
@@ -276,6 +305,13 @@ if __name__ == '__main__':
     ap.add_argument('--max_days', help="Maximum number of days for the plan to take.", default=365, required=False)
     ap.add_argument('--transit_fraction', help="Minimum transit fraction to be observable.", type=float, default=0.5,
                     required=False)
+    ap.add_argument('--baseline', help="Required baseline (in hours) for the observation.", type=float, default=0.5,
+                    required=False)
+    ap.add_argument('--since', help="yyyy-mm-dd date since when you want to start the plan (defaults to today).",
+                    type=str, default=None,
+                    required=False)
+    ap.add_argument('--error_sigma', help="Sigma to be used for epoch and period errors.", type=int, default=2,
+                    required=False)
     args = ap.parse_args()
     if args.observatories is None and (args.lat is None or args.lon is None or args.alt is None):
         raise ValueError("You either need to set the 'observatories' property or the lat, lon and alt.")
@@ -292,16 +328,21 @@ if __name__ == '__main__':
     fit_derived_results = pd.read_csv(object_dir + "/results/ns_derived_table.csv")
     fit_results = pd.read_csv(object_dir + "/results/ns_table.csv")
     candidates_count = len(fit_results[fit_results["#name"].str.contains("_period")])
+    alles = allesfitter.allesclass(object_dir)
+    since = Time.now() if args.since is None else Time(args.since, scale='utc')
+    percentile = 99.7 if args.error_sigma == 3 else 95 if args.error_sigma == 2 else 68
     for i in np.arange(0, candidates_count):
         period_row = fit_results[fit_results["#name"].str.contains("_period")].iloc[i]
         period = period_row["median"]
-        period_low_err = float(period_row["lower_error"])
-        period_up_err = float(period_row["upper_error"])
+        period_distribution = alles.posterior_params[period_row["#name"]]
+        period_low_err = period - np.percentile(period_distribution, 50 - percentile / 2)
+        period_up_err = np.percentile(period_distribution, 50 + percentile / 2) - period
         name = object_id + "_" + period_row["#name"].replace("_period", "")
         epoch_row = fit_results[fit_results["#name"].str.contains("_epoch")].iloc[i]
         epoch = epoch_row["median"].item()
-        epoch_low_err = float(epoch_row["lower_error"])
-        epoch_up_err = float(epoch_row["upper_error"])
+        epoch_distribution = alles.posterior_params[epoch_row["#name"]]
+        epoch_low_err = epoch - np.percentile(epoch_distribution, 50 - percentile / 2)
+        epoch_up_err = np.percentile(epoch_distribution, 50 + percentile / 2) - epoch
         duration_row = fit_derived_results[fit_derived_results["#property"].str.contains("Total transit duration")].iloc[i]
         duration = duration_row["value"].item()
         duration_low_err = float(duration_row["lower_error"])
@@ -310,12 +351,14 @@ if __name__ == '__main__':
         depth = depth_row["value"] * 1000
         depth_low_err = depth_row["lower_error"] * 1000
         depth_up_err = depth_row["upper_error"] * 1000
-        observatories_df, observables_df, plan_dir, images_dir = create_observation_observables(object_id, object_dir, name, epoch,
+        observatories_df, observables_df, alert_date, plan_dir, images_dir = create_observation_observables(object_id, object_dir,
+                                                                                                since, name, epoch,
                                                             epoch_low_err, epoch_up_err, period, period_low_err,
                                                             period_up_err, duration, args.observatories, args.tz, args.lat,
                                                             args.lon, args.alt, args.max_days, args.min_altitude,
-                                                            args.moon_min_dist, args.moon_max_dist, args.transit_fraction)
-        report = ObservationReport(observatories_df, observables_df, object_id, name, plan_dir, ra, dec,
+                                                            args.moon_min_dist, args.moon_max_dist, args.transit_fraction,
+                                                            args.baseline)
+        report = ObservationReport(observatories_df, observables_df, alert_date, object_id, name, plan_dir, ra, dec,
                                    epoch, epoch_low_err, epoch_up_err, period, period_low_err, period_up_err, duration,
                                    duration_low_err, duration_up_err, depth, depth_low_err, depth_up_err,
                                    args.transit_fraction, args.moon_min_dist, args.moon_max_dist, args.min_altitude,
