@@ -491,7 +491,7 @@ class Sherlock:
         transit_results = self.__identify_signals(sherlock_target, time, lcs, flux_err, star_info, transits_min_count,
                                                   wl, id_run, cadence, report, period_grid, detrend_source_period)
         signal_selection = sherlock_target.signal_score_selectors[sherlock_target.best_signal_algorithm]\
-            .select(transit_results, sherlock_target.snr_min, sherlock_target.detrend_method, wl)
+            .select(transit_results, sherlock_target.snr_min, sherlock_target.sde_min, sherlock_target.detrend_method, wl)
         logging.info(signal_selection.get_message())
         return transit_results, signal_selection
 
@@ -845,8 +845,8 @@ class Sherlock:
             clean_lcs = []
             for key, flux in enumerate(lcs):
                 flux[intransit] = np.nan
-                clean_time, clean_flux = tls.cleaned_array(time, flux)
-                clean_lcs.append(clean_flux)
+                clean_time = time
+                clean_lcs.append(flux)
         return clean_time, np.array(clean_lcs)
 
     def run_multiprocessing(self, n_processors, func, func_input):
