@@ -109,13 +109,13 @@ class TestsSherlock(unittest.TestCase):
         sherlock = Sherlock([])
         self.assertFalse(sherlock.use_ois)
         sherlock.run()
-        object_dir = "TIC181084752_FFI_all"
+        object_dir = "TIC181084752_FFI_[9]"
         self.assertFalse(os.path.exists(object_dir))
 
     def test_run(self):
-        run_dir = "TIC181804752_FFI_all"
+        run_dir = "TIC181804752_FFI_[9]"
         try:
-            Sherlock([SherlockTarget(MissionFfiIdObjectInfo("TIC 181804752", 'all', smooth_enabled=False,
+            Sherlock([SherlockTarget(MissionFfiIdObjectInfo("TIC 181804752", [9], smooth_enabled=False,
                                                             high_rms_enabled=False),
                                      detrends_number=1, max_runs=1)]).run()
             self.__assert_run_files(run_dir, assert_rms_mask=False)
@@ -123,9 +123,9 @@ class TestsSherlock(unittest.TestCase):
             self.__clean(run_dir)
 
     def test_run_with_rms_mask(self):
-        run_dir = "TIC181804752_FFI_all"
+        run_dir = "TIC181804752_FFI_[9]"
         try:
-            Sherlock([SherlockTarget(MissionFfiIdObjectInfo("TIC 181804752", 'all', high_rms_enabled=True),
+            Sherlock([SherlockTarget(MissionFfiIdObjectInfo("TIC 181804752", [9], high_rms_enabled=True),
                                      max_runs=1)]).run()
             self.__assert_run_files(run_dir)
         finally:
@@ -134,11 +134,11 @@ class TestsSherlock(unittest.TestCase):
     def test_run_with_explore(self):
         run_dir = None
         try:
-            Sherlock([SherlockTarget(MissionFfiIdObjectInfo("TIC 181804752", 'all', high_rms_enabled=True),
+            Sherlock([SherlockTarget(MissionFfiIdObjectInfo("TIC 181804752", [9], high_rms_enabled=True),
                                      detrends_number=1)], True).run()
-            run_dir = "TIC181804752_FFI_all"
+            run_dir = "TIC181804752_FFI_[9]"
             self.assertTrue(os.path.exists(run_dir))
-            self.assertTrue(os.path.exists(run_dir + "/Initial_Periodogram_TIC181804752_FFI_all.png"))
+            self.assertTrue(os.path.exists(run_dir + "/Periodogram_Initial_TIC181804752_FFI_[9].png"))
             self.assertFalse(os.path.exists(run_dir + "/1"))
         finally:
             self.__clean(run_dir)
@@ -162,7 +162,7 @@ class TestsSherlock(unittest.TestCase):
                                      detrends_number=1, max_runs=1)], False).run()
             run_dir = "EPIC249631677_FFI_all"
             self.assertTrue(os.path.exists(run_dir))
-            self.assertTrue(os.path.exists(run_dir + "/Initial_Periodogram_EPIC249631677_FFI_all.png"))
+            self.assertTrue(os.path.exists(run_dir + "/Periodogram_Initial_EPIC249631677_FFI_all.png"))
             self.assertTrue(os.path.exists(run_dir + "/1"))
         finally:
             self.__clean(run_dir)
@@ -170,7 +170,7 @@ class TestsSherlock(unittest.TestCase):
     def test_run_with_star_info(self):
         run_dir = None
         try:
-            Sherlock([SherlockTarget(MissionFfiIdObjectInfo("TIC 181804752", 'all', high_rms_enabled=True,
+            Sherlock([SherlockTarget(MissionFfiIdObjectInfo("TIC 181804752", [9], high_rms_enabled=True,
                                                             star_info=StarInfo(ld_coefficients=(0.15,0.25),
                                                                                teff=4000,
                                                                                lum=1.50, logg=0.15, radius=0.4,
@@ -178,11 +178,11 @@ class TestsSherlock(unittest.TestCase):
                                                                                mass=0.3, mass_min=0.05, mass_max=0.075,
                                                                                ra=13.132258, dec=64.334238)),
                                      detrends_number=1, max_runs=1)], True).run()
-            run_dir = "TIC181804752_FFI_all"
+            run_dir = "TIC181804752_FFI_[9]"
             self.assertTrue(os.path.exists(run_dir))
-            self.assertTrue(os.path.exists(run_dir + "/Initial_Periodogram_TIC181804752_FFI_all.png"))
+            self.assertTrue(os.path.exists(run_dir + "/Periodogram_Initial_TIC181804752_FFI_[9].png"))
             self.assertFalse(os.path.exists(run_dir + "/1"))
-            with open(run_dir + '/TIC181804752_FFI_all_report.log') as f:
+            with open(run_dir + '/TIC181804752_FFI_[9]_report.log') as f:
                 content = f.read()
                 self.assertTrue('mass = 0.3' in content)
                 self.assertTrue('mass_min = 0.25' in content)
@@ -200,13 +200,13 @@ class TestsSherlock(unittest.TestCase):
     def test_run_with_transit_customs(self):
         run_dir = None
         try:
-            sherlock = Sherlock([SherlockTarget(MissionFfiIdObjectInfo("TIC 181804752", 'all', high_rms_enabled=True),
+            sherlock = Sherlock([SherlockTarget(MissionFfiIdObjectInfo("TIC 181804752", [9], high_rms_enabled=True),
                                                 detrends_number=1, max_runs=1, oversampling=5.5, t0_fit_margin=0.09,
                                                 duration_grid_step=1.075, fit_method="bls",
                                                 best_signal_algorithm="quorum", quorum_strength=0.31)], False)\
                 .run()
-            run_dir = "TIC181804752_FFI_all"
-            with open(run_dir + '/TIC181804752_FFI_all_report.log') as f:
+            run_dir = "TIC181804752_FFI_[9]"
+            with open(run_dir + '/TIC181804752_FFI_[9]_report.log') as f:
                 content = f.read()
                 self.assertTrue('Fit method: box' in content)
                 self.assertTrue('Duration step: 1.075' in content)
@@ -220,14 +220,16 @@ class TestsSherlock(unittest.TestCase):
 
     def __assert_run_files(self, object_dir, assert_rms_mask=True):
         run_dir = object_dir + "/1"
-        periodogram_file = object_dir + "/Initial_Periodogram_TIC181804752_FFI_all.png"
-        rms_mask_file = object_dir + "/rms_mask/High_RMS_Mask_TIC181804752_FFI_all.png"
+        periodogram_file = object_dir + "/Periodogram_Initial_TIC181804752_FFI_[9].png"
+        periodogram_file1 = object_dir + "/Periodogram_Final_TIC181804752_FFI_[9].png"
+        rms_mask_file = object_dir + "/rms_mask/High_RMS_Mask_TIC181804752_FFI_[9].png"
         lc_file = object_dir + "/lc.csv"
-        report_file = object_dir + "/TIC181804752_FFI_all_report.log"
+        report_file = object_dir + "/TIC181804752_FFI_[9]_report.log"
         candidates_csv_file = object_dir + "/candidates.csv"
         try:
             self.assertTrue(os.path.exists(run_dir))
             self.assertTrue(os.path.isfile(periodogram_file))
+            self.assertTrue(os.path.isfile(periodogram_file1))
             if assert_rms_mask:
                 self.assertTrue(os.path.isfile(rms_mask_file))
             self.assertTrue(os.path.isfile(lc_file))
