@@ -5,89 +5,75 @@ import multiprocessing
 import shutil
 from multiprocessing import Pool
 import traceback
-import lightkurve
 import numpy as np
-import matplotlib
 import matplotlib.pyplot as plt
 import yaml
 from astroquery.mast import TesscutClass
 from lcbuilder.lcbuilder_class import LcBuilder
-from lightkurve import TessLightCurve
+from lightkurve import TessLightCurve, KeplerLightCurve
 from argparse import ArgumentParser
-from scipy import stats
-from sherlockpipe import constants as const
-from sherlockpipe import eleanor
-import six
 import sys
-import sherlockpipe.LATTE
-sys.modules['astropy.extern.six'] = six
-sys.modules['LATTE'] = sherlockpipe.LATTE
-
-matplotlib.use('Agg')
 import pandas as pd
 import os
 import ast
-from sherlockpipe.LATTE import LATTEutils, LATTEbrew
-from os import path
 import triceratops.triceratops as tr
-from matplotlib import cm, ticker
-from triceratops.likelihoods import (simulate_TP_transit, simulate_EB_transit)
-from triceratops.funcs import renorm_flux
-from astropy import constants
-from sherlockpipe.eleanor import maxsector
+from sherlockpipe.vet import Vetter
+from lcbuilder.eleanor import maxsector
 
-'''WATSON: Verboseless Vetting and Adjustments of Transits for Sherlock Objects of iNterest
-This class intends to provide a inspection and transit fitting tool for SHERLOCK Candidates.
-'''
-# get the system path
-syspath = str(os.path.abspath(LATTEutils.__file__))[0:-14]
-# ---------
-
-# --- IMPORTANT TO SET THIS ----
-out = 'pipeline'  # or TALK or 'pipeline'
-ttran = 0.1
-resources_dir = path.join(path.dirname(__file__))
 
 class Validator:
-    def __init__(self, object_dir):
+    """
+    This class intends to provide a statistical validation tool for SHERLOCK Candidates.
+    """
+    def __init__(self, object_dir, validation_dir):
         self.object_dir = os.getcwd() if object_dir is None else object_dir
-        self.data_dir = self.object_dir
-        self.validation_runs = 5
+        self.data_dir = validation_dir
 
-    def validate(self, candidate, cpus):
-        object_id = candidate["TICID"]
-        period = candidate.loc[candidate['TICID'] == object_id]['period'].iloc[0]
-        duration = candidate.loc[candidate['TICID'] == object_id]['duration'].iloc[0]
-        t0 = candidate.loc[candidate['TICID'] == object_id]['t0'].iloc[0]
-        transit_list = ast.literal_eval(((candidate.loc[candidate['TICID'] == object_id]['transits']).values)[0])
-        transit_depth = candidate.loc[candidate['TICID'] == object_id]['depth'].iloc[0]
-        rprs = candidate.loc[candidate['TICID'] == object_id]['rp_rs'].iloc[0]
-        candidate_row = candidate.iloc[0]
-        if candidate_row["number"] is None or np.isnan(candidate_row["number"]):
-            lc_file = "/" + candidate_row["lc"]
-        else:
-            lc_file = "/" + str(candidate_row["number"]) + "/lc_" + str(candidate_row["curve"]) + ".csv"
+    def validate(self, candidate, star, cpus, contrast_curve_file, bins=100, scenarios=5, sigma_mode="flux_err"):
+        """
+        @param candidate: a candidate dataframe containing TICID, period, duration, t0, transits, depth, rp_rs, number,
+        curve and sectors data.
+        @param star: the star dataframe.
+        @param cpus: the number of cpus to be used.
+        @param contrast_curve_file: the auxiliary contrast curve file to give more information to the validation engine.
+        @param bins: the number of bins to resize the light curve
+        @param scenarios: the number of scenarios to compute the validation and get the average
+        @param sigma_mode: whether to compute the sigma for the validation from the 'flux_err' or the 'binning'.
+        """
+        object_id = candidate["id"]
+        period = candidate.loc[candidate['id'] == object_id]['period'].iloc[0]
+        duration = candidate.loc[candidate['id'] == object_id]['duration'].iloc[0]
+        t0 = candidate.loc[candidate['id'] == object_id]['t0'].iloc[0]
+        transit_depth = candidate.loc[candidate['id'] == object_id]['depth'].iloc[0]
+        run = int(candidate.loc[candidate['id'] == object_id]['number'].iloc[0])
+        curve = int(candidate.loc[candidate['id'] == object_id]['curve'].iloc[0])
+        rp_rstar = candidate.loc[candidate['id'] == object_id]['rp_rs'].iloc[0]
+        a_rstar = candidate.loc[candidate['id'] == object_id]['a'].iloc[0] / star["R_star"]
+        logging.info("------------------")
+        logging.info("Candidate info")
+        logging.info("------------------")
+        logging.info("Period (d): %.2f", period)
+        logging.info("Epoch (d): %.2f", t0)
+        logging.info("Duration (min): %.2f", duration)
+        logging.info("Depth (ppt): %.2f", transit_depth)
+        logging.info("Run: %.0f", run)
+        logging.info("Detrend curve: %.0f", curve)
+        logging.info("Contrast curve file %s", contrast_curve_file)
+        lc_file = "/" + str(run) + "/lc_" + str(curve) + ".csv"
         lc_file = self.data_dir + lc_file
         try:
-            sectors_in = ast.literal_eval(str(((candidate.loc[candidate['TICID'] == object_id]['sectors']).values)[0]))
+            sectors_in = ast.literal_eval(str(((candidate.loc[candidate['id'] == object_id]['sectors']).values)[0]))
             if (type(sectors_in) == int) or (type(sectors_in) == float):
                 sectors = [sectors_in]
             else:
                 sectors = list(sectors_in)
         except:
             sectors = [0]
-        index = 0
-        validation_dir = self.data_dir + "/validation_" + str(index)
-        while os.path.exists(validation_dir) or os.path.isdir(validation_dir):
-            validation_dir = self.data_dir + "/validation_" + str(index)
-            index = index + 1
-        os.mkdir(validation_dir)
         self.data_dir = validation_dir
-        object_id = object_id[0]
-        mission, mission_prefix, id_int = LcBuilder().parse_object_info(object_id)
+        object_id = object_id.iloc[0]
         try:
-            self.execute_triceratops(cpus, validation_dir, mission, str(id_int), sectors, lc_file, transit_depth,
-                                     period, t0, duration)
+            Validator.execute_triceratops(cpus, validation_dir, object_id, sectors, lc_file, transit_depth,
+                                          period, t0, duration, rp_rstar, a_rstar, bins, scenarios, sigma_mode, contrast_curve_file)
         except Exception as e:
             traceback.print_exc()
         # try:
@@ -95,8 +81,9 @@ class Validator:
         # except Exception as e:
         #     traceback.print_exc()
 
-    def execute_triceratops(self, cpus, indir, mission, id_int, sectors, lc_file, transit_depth, period, t0,
-                            transit_duration):
+    @staticmethod
+    def execute_triceratops(cpus, indir, object_id, sectors, lc_file, transit_depth, period, t0,
+                            transit_duration, rp_rstar, a_rstar, bins, scenarios, sigma_mode, contrast_curve_file):
         """ Calculates probabilities of the signal being caused by any of the following astrophysical sources:
         TP No unresolved companion. Transiting planet with Porb around target star. (i, Rp)
         EB No unresolved companion. Eclipsing binary with Porb around target star. (i, qshort)
@@ -122,13 +109,19 @@ class Validator:
         when rounding to the nearest percent)
         @param cpus: number of cpus to be used
         @param indir: root directory to store the results
-        @param id_int: the tess object id for which the analysis will be run
+        @param id_int: the object id for which the analysis will be run
         @param sectors: the sectors of the tic
         @param lc_file: the light curve source file
         @param transit_depth: the depth of the transit signal (ppts)
         @param period: the period of the transit signal /days)
         @param t0: the t0 of the transit signal (days)
         @param transit_duration: the duration of the transit signal (minutes)
+        @param rp_rstar: radius of planet divided by radius of star
+        @param a_rstar: semimajor axis divided by radius of star
+        @param bins: the number of bins to average the folded curve
+        @param scenarios: the number of scenarios to validate
+        @param sigma_mode: the way to calculate the sigma for the validation ['flux_err' | 'binning']
+        @param contrast_curve_file: the auxiliary contrast curve file to give more information to the validation engine.
         """
         save_dir = indir + "/triceratops"
         if os.path.exists(save_dir):
@@ -136,18 +129,14 @@ class Validator:
         if not os.path.exists(save_dir):
             os.makedirs(save_dir)
         duration = transit_duration / 60 / 24
-        if mission != "TESS":
-            star_df = pd.read_csv(self.object_dir + "/params_star.csv")
-            star = eleanor.multi_sectors(coords=(star_df.at[0, "ra"], star_df.at[0, "dec"]), sectors='all',
-                                         tesscut_size=31, post_dir=const.USER_HOME_ELEANOR_CACHE)
-            if len(star) == 0 or star[0].tic is None:
-                raise ValueError("Can't validate target as there is no data from TESS FFIs")
-            tic = star[0].tic
-            sectors = np.array([target.sector for target in star])
-        else:
+        logging.info("----------------------")
+        logging.info("Validation procedures")
+        logging.info("----------------------")
+        logging.info("Pre-processing sectors")
+        mission, mission_prefix, id_int = LcBuilder().parse_object_info(object_id)
+        if mission == "TESS":
             sectors = np.array(sectors)
-            tic = id_int
-            sectors_cut = TesscutClass().get_sectors("TIC " + str(tic))
+            sectors_cut = TesscutClass().get_sectors("TIC " + str(id_int))
             sectors_cut = np.array([sector_row["sector"] for sector_row in sectors_cut])
             if len(sectors) != len(sectors_cut):
                 logging.warning("WARN: Some sectors were not found in TESSCUT")
@@ -164,45 +153,28 @@ class Validator:
             if len(sectors) == 0:
                 logging.warning("There are no available sectors to be validated, skipping TRICERATOPS.")
                 return save_dir, None, None
-        target = tr.target(ID=tic, sectors=sectors)
         logging.info("Will execute validation for sectors: " + str(sectors))
+        logging.info("Acquiring triceratops target")
+        target = tr.target(ID=id_int, mission=mission, sectors=sectors)
         # TODO allow user input apertures
-        tpfs = lightkurve.search_targetpixelfile("TIC " + str(tic), mission="TESS", cadence="short", sector=sectors.tolist())\
-            .download_all()
-        star = eleanor.multi_sectors(tic=tic, sectors=sectors, tesscut_size=11, post_dir=const.USER_HOME_ELEANOR_CACHE)
-        apertures = []
-        sector_num = 0
-        logging.info("Calculating validation masks")
-        for s in star:
-            if tpfs is None:
-                target_data = eleanor.TargetData(s, height=11, width=11)
-                pipeline_mask = target_data.aperture.astype(bool)
-                column = s.position_on_chip[0]
-                row = s.position_on_chip[1]
-            else:
-                tpf_idx = [data.sector if data.sector == s.sector else -1 for data in tpfs.data]
-                tpf = tpfs[np.where(tpf_idx > np.zeros(len(tpf_idx)))[0][0]]
-                pipeline_mask = tpfs[np.where(tpf_idx > np.zeros(len(tpf_idx)))[0][0]].pipeline_mask
-                column = tpf.column
-                row = tpf.row
-            pipeline_mask = np.transpose(pipeline_mask)
-            pipeline_mask_triceratops = np.zeros((len(pipeline_mask[0]), len(pipeline_mask[:][0]), 2))
-            for i in range(0, len(pipeline_mask[0])):
-                for j in range(0, len(pipeline_mask[:][0])):
-                    pipeline_mask_triceratops[i, j] = [column + i, row + j]
-            pipeline_mask_triceratops[~pipeline_mask] = None
-            aperture = []
-            for i in range(0, len(pipeline_mask_triceratops[0])):
-                for j in range(0, len(pipeline_mask_triceratops[:][0])):
-                    if not np.isnan(pipeline_mask_triceratops[i, j]).any():
-                        aperture.append(pipeline_mask_triceratops[i, j])
-            apertures.append(aperture)
-            logging.info("Saving validation mask plot for sector %s", s.sector)
-            target.plot_field(save=True, fname=save_dir + "/field_S" + str(s.sector), sector=s.sector,
-                            ap_pixels=aperture)
-            sector_num = sector_num + 1
-        apertures = np.array(apertures)
+        logging.info("Reading apertures from directory")
+        apertures = yaml.load(open(object_dir + "/apertures.yaml"), yaml.SafeLoader)
+        apertures = apertures["sectors"]
+        for sector, aperture in apertures.items():
+            target.plot_field(save=True, fname=save_dir + "/field_S" + str(sector), sector=sector, ap_pixels=aperture)
+        apertures = np.array([aperture for sector, aperture in apertures.items()])
         depth = transit_depth / 1000
+        if contrast_curve_file is not None:
+            logging.info("Reading contrast curve %s", contrast_curve_file)
+            plt.clf()
+            cc = pd.read_csv(contrast_curve_file, header=None)
+            sep, dmag = cc[0].values, cc[1].values
+            plt.plot(sep, dmag, 'k-')
+            plt.ylim(9, 0)
+            plt.ylabel("$\\Delta K_s$", fontsize=20)
+            plt.xlabel("separation ('')", fontsize=20)
+            plt.savefig(save_dir + "/contrast_curve.png")
+            plt.clf()
         logging.info("Calculating validation closest stars depths")
         target.calc_depths(depth, apertures)
         target.stars.to_csv(save_dir + "/stars.csv", index=False)
@@ -211,38 +183,37 @@ class Validator:
         lc_len = len(time)
         zeros_lc = np.zeros(lc_len)
         logging.info("Preparing validation light curve for target")
-        lc = TessLightCurve(time=time, flux=flux, flux_err=flux_err, quality=zeros_lc)
+        if mission == "TESS":
+            lc = TessLightCurve(time=time, flux=flux, flux_err=flux_err, quality=zeros_lc)
+        else:
+            lc = KeplerLightCurve(time=time, flux=flux, flux_err=flux_err, quality=zeros_lc)
         lc.extra_columns = []
-        lc = lc.fold(period=period, epoch_time=t0, normalize_phase=True)
-        folded_plot_range = duration / 2 / period * 5
-        inner_folded_range_args = np.where((0 - folded_plot_range < lc.time.value) & (lc.time.value < 0 + folded_plot_range))
-        lc = lc[inner_folded_range_args]
-        lc.time = lc.time * period
-        bin_means, bin_edges, binnumber = stats.binned_statistic(lc.time.value, lc.flux.value, statistic='mean', bins=500)
-        bin_width = (bin_edges[1] - bin_edges[0])
-        bin_centers = bin_edges[1:] - bin_width/2
-        lc.plot()
-        plt.title("TIC " + str(id_int))
+        fig, axs = plt.subplots(1, 1, figsize=(8, 4), constrained_layout=True)
+        axs, bin_centers, bin_means, bin_errs = Vetter.compute_phased_values_and_fill_plot(object_id, axs, lc, period,
+                                                                                           t0 + period / 2, depth,
+                                                                                           duration, rp_rstar, a_rstar,
+                                                                                           bins=bins)
         plt.savefig(save_dir + "/folded_curve.png")
-        plt.plot(bin_centers, bin_means)
-        plt.title("TIC " + str(id_int))
-        plt.xlabel("Time")
-        plt.ylabel("Flux")
-        plt.savefig(save_dir + "/folded_curve_binned.png")
-        sigma = np.nanmean(lc.flux_err)
+        plt.clf()
+        bin_centers = (bin_centers - 0.5) * period
+        logging.info("Sigma mode is %s", sigma_mode)
+        sigma = np.nanmean(bin_errs) if sigma_mode == 'binning' else np.nanmean(flux_err)
+        logging.info("Computed folded curve sigma = %s", sigma)
         logging.info("Preparing validation processes inputs")
         input_n_times = [ValidatorInput(save_dir, copy.deepcopy(target), bin_centers, bin_means, sigma, period, depth,
-                                        apertures, value)
-                         for value in range(0, self.validation_runs)]
-        validator = TriceratopsThreadValidator()
+                                        apertures, value, contrast_curve_file)
+                         for value in range(0, scenarios)]
+        thread_validator = TriceratopsThreadValidator()
         logging.info("Start validation processes")
         with Pool(processes=cpus) as pool:
-            validation_results = pool.map(validator.validate, input_n_times)
+            validation_results = pool.map(thread_validator.validate, input_n_times)
         logging.info("Finished validation processes")
         fpp_sum = 0
+        fpp2_sum = 0
+        fpp3_sum = 0
         nfpp_sum = 0
         probs_total_df = None
-        scenarios_num = len(validation_results[0][2])
+        scenarios_num = len(validation_results[0][4])
         star_num = np.zeros((5, scenarios_num))
         u1 = np.zeros((5, scenarios_num))
         u2 = np.zeros((5, scenarios_num))
@@ -254,40 +225,58 @@ class Validator:
         target.u2 = np.zeros(scenarios_num)
         target.fluxratio_EB = np.zeros(scenarios_num)
         target.fluxratio_comp = np.zeros(scenarios_num)
-        logging.info("Computing final probabilities from the %s scenarios", self.validation_runs)
+        logging.info("Computing final probabilities from the %s scenarios", scenarios)
         i = 0
-        for fpp, nfpp, probs_df, star_num_arr, u1_arr, u2_arr, fluxratio_EB_arr, fluxratio_comp_arr in validation_results:
-            if probs_total_df is None:
-                probs_total_df = probs_df
-            else:
-                probs_total_df = pd.concat((probs_total_df, probs_df))
-            fpp_sum = fpp_sum + fpp
-            nfpp_sum = nfpp_sum + nfpp
-            star_num[i] = star_num_arr
-            u1[i] = u1_arr
-            u2[i] = u2_arr
-            fluxratio_EB[i] = fluxratio_EB_arr
-            fluxratio_comp[i] = fluxratio_comp_arr
-            i = i + 1
-        for i in range(0, scenarios_num):
-            target.star_num[i] = np.mean(star_num[:, i])
-            target.u1[i] = np.mean(u1[:, i])
-            target.u2[i] = np.mean(u2[:, i])
-            target.fluxratio_EB[i] = np.mean(fluxratio_EB[:, i])
-            target.fluxratio_comp[i] = np.mean(fluxratio_comp[:, i])
         with open(save_dir + "/validation.csv", 'w') as the_file:
-            the_file.write("FPP,NFPP\n")
-            the_file.write(str(fpp_sum / self.validation_runs) + "," + str(nfpp_sum / self.validation_runs))
+            the_file.write("scenario,FPP,NFPP,FPP2,FPP3+\n")
+            for fpp, nfpp, fpp2, fpp3, probs_df, star_num_arr, u1_arr, u2_arr, fluxratio_EB_arr, fluxratio_comp_arr \
+                    in validation_results:
+                if probs_total_df is None:
+                    probs_total_df = probs_df
+                else:
+                    probs_total_df = pd.concat((probs_total_df, probs_df))
+                fpp_sum = fpp_sum + fpp
+                fpp2_sum = fpp2_sum + fpp2
+                fpp3_sum = fpp3_sum + fpp3
+                nfpp_sum = nfpp_sum + nfpp
+                star_num[i] = star_num_arr
+                u1[i] = u1_arr
+                u2[i] = u2_arr
+                fluxratio_EB[i] = fluxratio_EB_arr
+                fluxratio_comp[i] = fluxratio_comp_arr
+                the_file.write(str(i) + "," + str(fpp) + "," + str(nfpp) + "," + str(fpp2) + "," + str(fpp3) + "\n")
+                i = i + 1
+            for i in range(0, scenarios_num):
+                target.star_num[i] = np.mean(star_num[:, i])
+                target.u1[i] = np.mean(u1[:, i])
+                target.u2[i] = np.mean(u2[:, i])
+                target.fluxratio_EB[i] = np.mean(fluxratio_EB[:, i])
+                target.fluxratio_comp[i] = np.mean(fluxratio_comp[:, i])
+            fpp_sum = fpp_sum / scenarios
+            nfpp_sum = nfpp_sum / scenarios
+            fpp2_sum = fpp2_sum / scenarios
+            fpp3_sum = fpp3_sum / scenarios
+            logging.info("---------------------------------")
+            logging.info("Final probabilities computed")
+            logging.info("---------------------------------")
+            logging.info("FPP=%s", fpp_sum)
+            logging.info("NFPP=%s", nfpp_sum)
+            logging.info("FPP2(Lissauer et al, 2012)=%s", fpp2_sum)
+            logging.info("FPP3+(Lissauer et al, 2012)=%s", fpp3_sum)
+            the_file.write("MEAN" + "," + str(fpp_sum) + "," + str(nfpp_sum) + "," + str(fpp2_sum) + "," +
+                           str(fpp3_sum))
         probs_total_df = probs_total_df.groupby("scenario", as_index=False).mean()
-        probs_total_df["scenario"] = pd.Categorical(probs_total_df["scenario"], ["TP", "EB", "EBx2P", "PTP", "PEB", "PEBx2P",
-                                                                         "STP", "SEB", "SEBx2P", "DTP", "DEB", "DEBx2P",
-                                                                         "BTP", "BEB", "BEBx2P", "NTP", "NEB", "NEBx2P"])
+        probs_total_df["scenario"] = pd.Categorical(probs_total_df["scenario"], ["TP", "EB", "EBx2P", "PTP", "PEB",
+                                                                                 "PEBx2P", "STP", "SEB", "SEBx2P",
+                                                                                 "DTP", "DEB", "DEBx2P", "BTP", "BEB",
+                                                                                 "BEBx2P", "NTP", "NEB", "NEBx2P"])
         probs_total_df = probs_total_df.sort_values("scenario")
         probs_total_df.to_csv(save_dir + "/validation_scenarios.csv", index=False)
         target.probs = probs_total_df
         # target.plot_fits(save=True, fname=save_dir + "/scenario_fits", time=lc.time.value, flux_0=lc.flux.value,
         #                  flux_err_0=sigma)
-        return save_dir, star[0].coords[0], star[0].coords[1]
+        return save_dir
+
 
     # def execute_vespa(self, cpus, indir, object_id, sectors, lc_file, transit_depth, period, epoch, duration, rprs):
     #     vespa_dir = indir + "/vespa/"
@@ -445,167 +434,39 @@ class Validator:
     #         bands=None
     #     )
 
-    @staticmethod
-    def plot_fits(target, save_file, time: np.ndarray, flux_0: np.ndarray, sigma_0: float):
-        """
-        Plots light curve for best fit instance of each scenario.
-        Args:
-            time (numpy array): Time of each data point
-                                [days from transit midpoint].
-            flux_0 (numpy array): Normalized flux of each data point.
-            sigma_0 (numpy array): Uncertainty of flux.
-        """
-        scenario_idx = target.probs[target.probs["ID"] != 0].index.values
-        df = target.probs[target.probs["ID"] != 0]
-        star_num = target.star_num[target.probs["ID"] != 0]
-        u1s = target.u1[target.probs["ID"] != 0]
-        u2s = target.u2[target.probs["ID"] != 0]
-        fluxratios_EB = target.fluxratio_EB[target.probs["ID"] != 0]
-        fluxratios_comp = target.fluxratio_comp[target.probs["ID"] != 0]
-
-        model_time = np.linspace(min(time), max(time), 100)
-
-        f, ax = plt.subplots(
-            len(df)//3, 3, figsize=(12, len(df)//3*4), sharex=True
-            )
-        G = constants.G.cgs.value
-        M_sun = constants.M_sun.cgs.value
-        for i in range(len(df)//3):
-            for j in range(3):
-                if i == 0:
-                    k = j
-                else:
-                    k = 3*i+j
-                # subtract flux from other stars in the aperture
-                idx = np.argwhere(
-                    target.stars["ID"].values == str(df["ID"].values[k])
-                    )[0, 0]
-                flux, sigma = renorm_flux(
-                    flux_0, sigma_0, target.stars["fluxratio"].values[idx]
-                    )
-                # all TPs
-                if j == 0:
-                    if star_num[k] == 1:
-                        comp = False
-                    else:
-                        comp = True
-                    a = (
-                        (G*df["M_s"].values[k]*M_sun)/(4*np.pi**2)
-                        * (df['P_orb'].values[k]*86400)**2
-                        )**(1/3)
-                    u1, u2 = u1s[k], u2s[k]
-                    best_model = simulate_TP_transit(
-                        model_time,
-                        df['R_p'].values[k], df['P_orb'].values[k],
-                        df['inc'].values[k], a, df["R_s"].values[k],
-                        u1, u2, fluxratios_comp[k], comp
-                        )
-                # all small EBs
-                elif j == 1:
-                    if star_num[k] == 1:
-                        comp = False
-                    else:
-                        comp = True
-                    mass = df["M_s"].values[k] + df["M_EB"].values[k]
-                    a = (
-                        (G*mass*M_sun)/(4*np.pi**2)
-                        * (df['P_orb'].values[k]*86400)**2
-                        )**(1/3)
-                    u1, u2 = u1s[k], u2s[k]
-                    best_model = simulate_EB_transit(
-                        model_time,
-                        df["R_EB"].values[k], fluxratios_EB[k],
-                        df['P_orb'].values[k], df['inc'].values[k],
-                        a, df["R_s"].values[k], u1, u2,
-                        fluxratios_comp[k], comp
-                        )[0]
-                # all twin EBs
-                elif j == 2:
-                    if star_num[k] == 1:
-                        comp = False
-                    else:
-                        comp = True
-                    mass = df["M_s"].values[k] + df["M_EB"].values[k]
-                    a = (
-                        (G*mass*M_sun)/(4*np.pi**2)
-                        * (df['P_orb'].values[k]*86400)**2
-                        )**(1/3)
-                    u1, u2 = u1s[k], u2s[k]
-                    best_model = simulate_EB_transit(
-                        model_time,
-                        df["R_EB"].values[k], fluxratios_EB[k],
-                        df['P_orb'].values[k], df['inc'].values[k],
-                        a, df["R_s"].values[k], u1, u2,
-                        fluxratios_comp[k], comp
-                        )[0]
-
-                y_formatter = ticker.ScalarFormatter(useOffset=False)
-                ax[i, j].yaxis.set_major_formatter(y_formatter)
-                ax[i, j].errorbar(
-                    time, flux, sigma, fmt=".",
-                    color="blue", alpha=0.1, zorder=0
-                    )
-                ax[i, j].plot(
-                    model_time, best_model, "k-", lw=5, zorder=2
-                    )
-                ax[i, j].set_ylabel("normalized flux", fontsize=12)
-                ax[i, j].annotate(
-                    str(df["ID"].values[k]), xy=(0.05, 0.92),
-                    xycoords="axes fraction", fontsize=12
-                    )
-                ax[i, j].annotate(
-                    str(df["scenario"].values[k]), xy=(0.05, 0.05),
-                    xycoords="axes fraction", fontsize=12
-                    )
-        ax[len(df)//3-1, 0].set_xlabel(
-            "days from transit center", fontsize=12
-            )
-        ax[len(df)//3-1, 1].set_xlabel(
-            "days from transit center", fontsize=12
-            )
-        ax[len(df)//3-1, 2].set_xlabel(
-            "days from transit center", fontsize=12
-            )
-        plt.tight_layout()
-        if save_file is not None:
-            plt.savefig(save_file)
-        plt.show()
-        return
-
-    def show_candidates(self):
-        self.candidates = pd.read_csv(self.object_dir + "/candidates.csv")
-        self.candidates.index = np.arange(1, len(self.candidates) + 1)
-        logging.info("Suggested candidates are:")
-        logging.info(self.candidates.to_markdown(index=True))
-        pass
-
-    def demand_candidate_selection(self):
-        user_input = input("Select candidate number to be examined and fit: ")
-        if user_input.startswith("q"):
-            raise SystemExit("User quitted")
-        self.candidate_selection = int(user_input)
-        if self.candidate_selection < 1 or self.candidate_selection > len(self.candidates.index):
-            raise SystemExit("User selected a wrong candidate number.")
-        self.data_dir = self.object_dir + "/" + str(self.candidate_selection)
 
 class TriceratopsThreadValidator:
+    """
+    Used to run a single scenario validation with TRICERATOPS
+    """
     def __init__(self) -> None:
         super().__init__()
 
     def validate(self, input):
+        """
+        Computes the input scenario FPP and NFPP. In addition, FPP2 and FPP3+, from the probability boost proposed in
+        Lissauer et al. (2012) eq. 8 and 9 for systems where one or more planets have already been confirmed, are also
+        provided just in case they are useful so they don't need to be manually calculated.
+        @param input: ValidatorInput
+        @return: the FPP values, the probabilities dataframe and additional target values.
+        """
         input.target.calc_depths(tdepth=input.depth, all_ap_pixels=input.apertures)
-        input.target.calc_probs(time=input.time, flux_0=input.flux, flux_err_0=input.sigma, P_orb=input.period)
-        with open(input.save_dir + "/validation_" + str(input.run) + ".csv", 'w') as the_file:
-            the_file.write("FPP,NFPP\n")
-            the_file.write(str(input.target.FPP) + "," + str(input.target.NFPP))
+        input.target.calc_probs(time=input.time, flux_0=input.flux, flux_err_0=input.sigma, P_orb=input.period,
+                                contrast_curve_file=input.contrast_curve, parallel=True)
+        fpp2 = 1 - 25 * (1 - input.target.FPP) / (25 * (1 - input.target.FPP) + input.target.FPP)
+        fpp3 = 1 - 50 * (1 - input.target.FPP) / (50 * (1 - input.target.FPP) + input.target.FPP)
         input.target.probs.to_csv(input.save_dir + "/validation_" + str(input.run) + "_scenarios.csv", index=False)
         input.target.plot_fits(save=True, fname=input.save_dir + "/scenario_" + str(input.run) + "_fits",
                          time=input.time, flux_0=input.flux, flux_err_0=input.sigma)
-        return input.target.FPP, input.target.NFPP, input.target.probs, input.target.star_num, input.target.u1, \
-               input.target.u2, input.target.fluxratio_EB, input.target.fluxratio_comp
+        return input.target.FPP, input.target.NFPP, fpp2, fpp3, input.target.probs, input.target.star_num, \
+               input.target.u1, input.target.u2, input.target.fluxratio_EB, input.target.fluxratio_comp
+
 
 class ValidatorInput:
-    def __init__(self, save_dir, target, time, flux, sigma, period, depth, apertures, run):
+    """
+    Wrapper class for input arguments of TriceratopsThreadValidator.
+    """
+    def __init__(self, save_dir, target, time, flux, sigma, period, depth, apertures, run, contrast_curve):
         self.save_dir = save_dir
         self.target = target
         self.time = time
@@ -615,17 +476,34 @@ class ValidatorInput:
         self.depth = depth
         self.apertures = apertures
         self.run = run
+        self.contrast_curve = contrast_curve
+
 
 if __name__ == '__main__':
-    ap = ArgumentParser(description='Vetting of Sherlock objects of interest')
+    ap = ArgumentParser(description='Validation of Sherlock objects of interest')
     ap.add_argument('--object_dir', help="If the object directory is not your current one you need to provide the "
                                          "ABSOLUTE path", required=False)
     ap.add_argument('--candidate', type=int, default=None, help="The candidate signal to be used.", required=False)
     ap.add_argument('--properties', help="The YAML file to be used as input.", required=False)
     ap.add_argument('--cpus', type=int, default=None, help="The number of CPU cores to be used.", required=False)
+    ap.add_argument('--bins', type=int, default=100, help="The number of bins to be used for the folded curve "
+                                                          "validation.", required=False)
+    ap.add_argument('--sigma_mode', type=str, default='flux_err', help="The way to calculate the sigma value for the "
+                                                                       "validation. [flux_err|binning]", required=False)
+    ap.add_argument('--scenarios', type=int, default=5, help="The number of scenarios to be used for the validation",
+                    required=False)
+    ap.add_argument('--contrast_curve', type=str, default=None, help="The contrast curve in csv format.",
+                    required=False)
     args = ap.parse_args()
-    validator = Validator(args.object_dir)
-    file_dir = validator.object_dir + "/validation.log"
+    index = 0
+    object_dir = os.getcwd() if args.object_dir is None else args.object_dir
+    validation_dir = object_dir + "/validation_" + str(index)
+    while os.path.exists(validation_dir) or os.path.isdir(validation_dir):
+        validation_dir = object_dir + "/validation_" + str(index)
+        index = index + 1
+    os.mkdir(validation_dir)
+    validator = Validator(object_dir, validation_dir)
+    file_dir = validation_dir + "/validation.log"
     if os.path.exists(file_dir):
         os.remove(file_dir)
     formatter = logging.Formatter('%(message)s')
@@ -642,23 +520,25 @@ if __name__ == '__main__':
     handler.setFormatter(formatter)
     logger.addHandler(handler)
     logging.info("Starting validation")
+    star_df = pd.read_csv(validator.object_dir + "/params_star.csv")
     if args.candidate is None:
+        logging.info("Reading validation input from properties file: %s", args.properties)
         user_properties = yaml.load(open(args.properties), yaml.SafeLoader)
-        candidate = pd.DataFrame(columns=['id', 'transits', 'sectors', 'FFI'])
+        candidate = pd.DataFrame(columns=['id', 'period', 'depth', 't0', 'sectors', 'ffi', 'number', 'lc'])
         candidate = candidate.append(user_properties, ignore_index=True)
-        cpus = user_properties["settings"]["cpus"]
+        candidate['id'] = star_df.iloc[0]["obj_id"]
     else:
         candidate_selection = int(args.candidate)
         candidates = pd.read_csv(validator.object_dir + "/candidates.csv")
         if candidate_selection < 1 or candidate_selection > len(candidates.index):
             raise SystemExit("User selected a wrong candidate number.")
-        candidates = candidates.rename(columns={'Object Id': 'TICID'})
+        candidates = candidates.rename(columns={'Object Id': 'id'})
         candidate = candidates.iloc[[candidate_selection - 1]]
         candidate['number'] = [candidate_selection]
         validator.data_dir = validator.object_dir
         logging.info("Selected signal number " + str(candidate_selection))
-        if args.cpus is None:
-            cpus = multiprocessing.cpu_count() - 1
-        else:
-            cpus = args.cpus
-    validator.validate(candidate, cpus)
+    if args.cpus is None:
+        cpus = multiprocessing.cpu_count() - 1
+    else:
+        cpus = args.cpus
+    validator.validate(candidate, star_df.iloc[0], cpus, args.contrast_curve, args.bins, args.scenarios, args.sigma_mode)
