@@ -16,6 +16,7 @@ import matplotlib.pyplot as plt
 import matplotlib.gridspec as gridspec
 import yaml
 from lcbuilder.constants import CUTOUT_SIZE
+from lcbuilder.photometry.aperture_extractor import ApertureExtractor
 from lightkurve import TessLightCurve
 from matplotlib.colorbar import Colorbar
 from matplotlib import patches
@@ -348,7 +349,7 @@ class Vetter:
             logging.info("Not plotting single transit for T0=%.2f as the data is empty", transit_time)
 
     @staticmethod
-    def plot_folded_curve(file_dir, id, lc, period, epoch, duration, depth, rp, rstar, a):
+    def plot_folded_curve(file_dir, id, lc, period, epoch, duration, depth, rp_rstar, a_rstar):
         """
         Plots the phase-folded curve of the candidate for period, 2 * period and period / 2.
         @param file_dir: the directory to store the plot
@@ -367,16 +368,16 @@ class Vetter:
         #TODO bins = None for FFI
         bins = 100
         Vetter.compute_phased_values_and_fill_plot(id, axs[0][0], lc, period, epoch + period / 2, depth, duration,
-                                                   rp, rstar, a, bins=bins)
-        Vetter.compute_phased_values_and_fill_plot(id, axs[0][1], lc, period, epoch, depth, duration, rp, rstar, a, bins=bins)
+                                                   rp_rstar, a_rstar, bins=bins)
+        Vetter.compute_phased_values_and_fill_plot(id, axs[0][1], lc, period, epoch, depth, duration, rp_rstar, a_rstar, bins=bins)
         period = 2 * period
         Vetter.compute_phased_values_and_fill_plot(id, axs[1][0], lc, period, epoch + period / 2, depth, duration,
-                                                   rp, rstar, a, bins=bins)
-        Vetter.compute_phased_values_and_fill_plot(id, axs[1][1], lc, period, epoch, depth, duration, rp, rstar, a, bins=bins)
+                                                   rp_rstar, a_rstar, bins=bins)
+        Vetter.compute_phased_values_and_fill_plot(id, axs[1][1], lc, period, epoch, depth, duration, rp_rstar, a_rstar, bins=bins)
         period = period / 4
         Vetter.compute_phased_values_and_fill_plot(id, axs[2][0], lc, period, epoch + period / 2, depth, duration,
-                                                   rp, rstar, a, bins=bins)
-        Vetter.compute_phased_values_and_fill_plot(id, axs[2][1], lc, period, epoch, depth, duration, rp, rstar, a, bins=bins)
+                                                   rp_rstar, a_rstar, bins=bins)
+        Vetter.compute_phased_values_and_fill_plot(id, axs[2][1], lc, period, epoch, depth, duration, rp_rstar, a_rstar, bins=bins)
         plt.savefig(file_dir + "/odd_even_folded_curves.png", dpi=200)
         fig.clf()
         plt.close(fig)
@@ -476,7 +477,7 @@ class Vetter:
             # get the transit time list
             period = df.loc[df['id'] == tic]['period'].iloc[0]
             t0 = df.loc[df['id'] == tic]['t0'].iloc[0]
-            rp_rstar = df.loc[df['id'] == tic]['rp_rstar'].iloc[0]
+            rp_rstar = df.loc[df['id'] == tic]['rp_rs'].iloc[0]
             a_rstar = df.loc[df['id'] == tic]['a'].iloc[0] / star["R_star"]
             duration = df.loc[df['id'] == tic]['duration'].iloc[0]
             depth = df.loc[df['id'] == tic]['depth'].iloc[0]
@@ -523,6 +524,15 @@ class Vetter:
                 except Exception as e1:
                     traceback.print_exc()
 
+    @staticmethod
+    def plot_tpf(tpf, sector, aperture, dir):
+        logging.info("Plotting FOV curves for sector %.0f", sector)
+        if not os.path.exists(dir):
+            os.mkdir(dir)
+        tpf.plot_pixels(aperture_mask=aperture)
+        plt.savefig(dir + "/Flux_pixels[" + str(sector) + "].png")
+        plt.close()
+
     @staticmethod
     def vetting_field_of_view(indir, mission, tic, cadence, ra, dec, sectors, source, apertures):
         """
@@ -576,6 +586,9 @@ class Vetter:
                                    extent=[column, column + ny, row, row + nx], origin='lower', zorder=0)
                 aperture = apertures[tpf.sector]
                 aperture = aperture if isinstance(aperture, np.ndarray) else np.array(aperture)
+                aperture_boolean = ApertureExtractor.from_pixels_to_boolean_mask(aperture, column, row, CUTOUT_SIZE,
+                                                                                 CUTOUT_SIZE)
+                Vetter.plot_tpf(tpf, tpf.sector, aperture_boolean, indir)
                 maskcolor = 'salmon'
                 logging.info("    --> Using SHERLOCK aperture for sector %s...", tpf.sector)
                 if aperture is not None:
